import MarkdownIt from 'markdown-it';
import hljs from 'highlight.js';
import MarkdownItContainer from 'markdown-it-container';
import MdAbbr from 'markdown-it-abbr';
import MdDefList from 'markdown-it-deflist';
import MdEmoji from 'markdown-it-emoji';
import MdFootnote from 'markdown-it-footnote';
import MdMark from 'markdown-it-mark';
import MdSub from 'markdown-it-sub';
import MdSup from 'markdown-it-sup';
import MdFigure from 'markdown-it-implicit-figures';
import MdAnchor from 'markdown-it-anchor';
import LazyHeaders from 'markdown-it-lazy-headers';
import InlineTokensIterator from 'markdown-it-for-inline';
import MarkdownTaskLists from 'markdown-it-task-lists';
import MarkdownItAttrs from 'markdown-it-attrs';
import { defaultTaskListsOption, defaultMarkdownItOption as defaultOption, defaultLinkifyOptions, anchorOption, } from './option';
import { TocPlugin } from './plugins/toc';
import { HighlightCodeLinesPlugin, LineNumberAttrRegex, } from './plugins/highlight-code-lines.plugin';
import { MetaPlugin } from './plugins/meta';
import { UniqueCustomIdAttrs } from './plugins/unique-custom-id';
import { ImageSizePlugin, MultilineBlockquotePlugin, simpleMathPlugin, diagramPlugin, HtmlFilterPlugin, } from './plugins';
const mdUtils = new MarkdownIt().utils;
const defaultCodeHighlight = (str, lang) => {
    if (lang && hljs.getLanguage(lang)) {
        try {
            const result = hljs.highlight(lang, str, true);
            return `<pre class="hljs"><code data-lang="${result.language}">${result.value}</code></pre>`;
        }
        catch (_a) { }
    }
    return `<pre class="hljs"><code>${mdUtils.escapeHtml(str)}</code></pre>`;
};
const addLinkAttributes = [
    InlineTokensIterator,
    'url_new_win',
    'link_open',
    (tokens, idx) => {
        const token = tokens[idx];
        const href = token.attrGet('href');
        if (!href) {
            return;
        }
        if (!href.startsWith('#')) {
            token.attrPush(['target', '_blank']);
        }
    },
];
const markdownItFactory = ({ html, codeHighlight, taskListsOption, math, disableRules, linkify: linkifyOption, preserveLineBreaks, enableMarkdownAttrs, showFrontMatter, enableImageSize, implicitFigures, } = defaultOption) => {
    const mdOpt = {
        html,
        breaks: preserveLineBreaks !== null && preserveLineBreaks !== void 0 ? preserveLineBreaks : true,
        linkify: true,
    };
    taskListsOption !== null && taskListsOption !== void 0 ? taskListsOption : (taskListsOption = defaultTaskListsOption);
    linkifyOption = Object.assign({}, defaultLinkifyOptions, linkifyOption !== null && linkifyOption !== void 0 ? linkifyOption : defaultLinkifyOptions);
    if (codeHighlight) {
        mdOpt.highlight = defaultCodeHighlight;
    }
    const md = new MarkdownIt(mdOpt)
        .use(MarkdownItContainer)
        .use(diagramPlugin)
        .use(MdAnchor, anchorOption)
        .use(TocPlugin)
        .use(MdAbbr)
        .use(MdDefList)
        .use(MdEmoji)
        .use(MdFootnote)
        .use(MdMark)
        .use(MdSub)
        .use(MdSup)
        .use(HighlightCodeLinesPlugin)
        .use(LazyHeaders)
        .use(MarkdownTaskLists, taskListsOption)
        .use(HtmlFilterPlugin)
        .use(...addLinkAttributes)
        .use(MultilineBlockquotePlugin)
        .use(MetaPlugin, { show: showFrontMatter !== null && showFrontMatter !== void 0 ? showFrontMatter : true });
    if (enableMarkdownAttrs !== null && enableMarkdownAttrs !== void 0 ? enableMarkdownAttrs : true) {
        md.use(MarkdownItAttrs, {
            allowedAttributes: ['id', 'class', LineNumberAttrRegex],
            disableParagraphBlock: true,
        }).use(UniqueCustomIdAttrs);
    }
    if (math) {
        md.use(simpleMathPlugin);
    }
    if (disableRules && disableRules.length > 0) {
        md.disable(disableRules);
    }
    if (enableImageSize == null || enableImageSize) {
        md.use(ImageSizePlugin);
    }
    const { normalize: defaultNormalize } = md.linkify.set(linkifyOption);
    if (linkifyOption.fuzzyLink && linkifyOption.fuzzyLinkUseHttps) {
        md.linkify.normalize = ((match) => {
            if (!match.schema) {
                match.url = `https://${match.url}`;
                return;
            }
            defaultNormalize(match);
        });
    }
    if (implicitFigures) {
        md.use(MdFigure, {
            figcaption: true,
            keepAlt: true,
            lazyLoading: true,
        });
    }
    return md;
};
export { markdownItFactory, MarkdownIt };
export * from './option';
export { MultilineBlockquotePlugin, MultilineBlockquoteRule, HighlightCodeLinesPlugin, } from './plugins';
//# sourceMappingURL=index.js.map